@@ -1,12 +1,9 @@
 # TODO: (TEST) write goddamn tests!
-# TODO: (FEAT) implement Apache Formatter
 # TODO: (FEAT) implement Tomcat Formatter
-# TODO: (FEAT) support for multiple formatters on same run
-# TODO: (FEAT) support for multiple transports on same run
 # TODO: (FEAT) support for different statistical distribution
-# TODO: (FEAT) support batch mode
 # TODO: (FEAT) support throughput testing thru http://linux.die.net/man/1/pv
 # TODO: (FEAT) support randomizing data fields (not only the data itself)
+# TODO: (FEAT) support infinite messages
 
 
 import pylog_config
@@ -31,11 +28,11 @@ DEFAULT_BASE_LOGGING_LEVEL = logging.INFO
 DEFAULT_VERBOSE_LOGGING_LEVEL = logging.DEBUG
 
 DEFAULT_CONFIG_FILE = 'config.py'
-DEFAULT_GAP = 0.01
+DEFAULT_GAP = float(0.01)
 DEFAULT_NUMBER_OF_MESSAGES = 100
 
-DEFAULT_TRANSPORT = 'FileTransport'
-DEFAULT_FORMAT = 'CustomFormatter'
+DEFAULT_TRANSPORT = 'File'
+DEFAULT_FORMATTER = 'Custom'
 
 
 def init_logger(base_level=DEFAULT_BASE_LOGGING_LEVEL,
@@ -115,7 +112,7 @@ def _import_config(config_file):
             config_file)[0])).GENERATOR
     except ImportError:
         lgr.warning('config file not found, assuming defaults.')
-        return None
+        raise PylogError('missing config file')
 
 
 def get_current_time():
@@ -143,7 +140,7 @@ def calculate_throughput(elapsed_time, messages):
         return 'Unknown', seconds
 
 
-def send(instance, client, format, format_config, messages, gap, batch):
+def send(instance, client, formatter, format_config, messages, gap, batch):
     """sends logs and prints the time it took to send all logs
 
     :param instance: transport class instance
@@ -157,7 +154,12 @@ def send(instance, client, format, format_config, messages, gap, batch):
     message_count = 0
     lgr.debug('configuring formatter...')
     # get formatter instance
-    formatter_instance = getattr(forms, format)(format_config)
+    if hasattr(forms, formatter):
+        formatter_instance = getattr(forms, formatter)(format_config)
+    else:
+        lgr.error('could not find formatter: {}. please make sure the '
+                  'formatter you\'re calling exists.'.format(formatter))
+        raise PylogError('missing formatter')
     # and get the current time
     start_time = get_current_time()
     lgr.debug('start time is: {}'.format(start_time))
@@ -205,18 +207,21 @@ def send(instance, client, format, format_config, messages, gap, batch):
 def config_transport(transports, transport, transport_config):
     """returns a configured instance and client for the transport
 
+    :param transports: transport classes to choose from.
     :param string transport: transport to use
     :param dict transport_config: transport configuration
     """
     lgr.debug('configuring transport...')
     # get transport instance
-    try:
-        instance = getattr(transports, transport)(transport_config)
-    except AttributeError:
+    if hasattr(transports, transport):
+        transport_instance = getattr(transports, transport)(transport_config)
+    else:
+        lgr.error('could not find transport: {}. please make sure the '
+                  'transport you\'re calling exists.'.format(transport))
         raise PylogError('missing transport')
     # get logging client
-    client = instance.configure()
-    return instance, client
+    client = transport_instance.configure()
+    return transport_instance, client
 
 
 def generator(config=None, transport=None, formatter=None, gap=None,
@@ -230,47 +235,50 @@ def generator(config=None, transport=None, formatter=None, gap=None,
     :param string formatter: formatter to use
     :param float gap: gap in seconds between 2 messages
     :param int messages: number of messages to send
+    :param int batch: number of messages to stack before sending
+    :param bool verbose: sets verbose state for internal logging.
     """
-    # TODO: DEPRECATE? maybe we do want multiple formats.. ehh..
-    def _create_formats_list(formats, formats_dict):
-        formats = []
-        if formats:
-            formats = filter(None, (formats or "").split(','))
-        else:
-            for format, values in formats_dict.items():
-                formats.append(format)
-        return formats
-
-    # set defaults for basic pylog configuration
-    if not transport:
-        transport = DEFAULT_TRANSPORT
-        lgr.info('transport not specified. assuming: {}'.format(transport))
-    if not formatter:
-        formatter = DEFAULT_FORMAT
-        lgr.info('formatter not specified. assuming: {}'.format(formatter))
-    if not gap:
-        gap = DEFAULT_GAP
-        lgr.info('gap not specified, assuming: {}'.format(gap))
-    if not messages:
-        messages = DEFAULT_NUMBER_OF_MESSAGES
-        lgr.info('message count not specified. assuming: {}'.format(messages))
-    batch = int(batch) if batch else 1
-    if batch > int(messages):
-        raise PylogError('batch number larger than total amount of messages')
     # set verbosity level for internal logging
     _set_global_verbosity_level(verbose)
+    # set params for basic pylog configuration
     # import config file with formatter and transport config
-    config = _import_config(config)
+    config = _import_config(config) if config else {}
+    transport = transport if transport else DEFAULT_TRANSPORT
+    formatter = formatter if formatter else DEFAULT_FORMATTER
+    gap = float(gap) if gap else DEFAULT_GAP
+    messages = int(messages) if messages else DEFAULT_NUMBER_OF_MESSAGES
+    batch = int(batch) if batch else 1
     # declare transport and formatter configuration. will assume defaults
     # if config file wasn't imported.
-    transport_config = config['transports'][transport] if config else {}
-    formatter_config = config['formatters'][formatter] if config else {}
-    # configure transport instance and client
-    instance, client = config_transport(trans, transport, transport_config)
-    # log messages
-    send(instance, client, formatter, formatter_config,
-         int(messages), float(gap), batch)
-    # maybe closing a connection to the host is required...
+    # TODO: (IMPRV) move config to different function.
+    transports_config = config.get('transports', {}) \
+        if config else {}
+    formatters_config = config.get('formatters', {}) \
+        if config else {}
+    transport_config = transports_config.get(transport, {}) \
+        if transports_config.get(transport) else {}
+    formatter_config = formatters_config.get(formatter, {}) \
+        if formatters_config.get(formatter) else {}
+    transport = transport_config.get('type', transport) \
+        if transport_config else transport
+    formatter = formatter_config.get('type', formatter) \
+        if formatter_config else formatter
+    # configure transport class instance and logging client
+    lgr.debug('transport: {}'.format(transport))
+    lgr.debug('formatter: {}'.format(formatter))
+    lgr.debug('gap: {}'.format(gap))
+    lgr.debug('message count: {}'.format(messages))
+    # well.. you can't have that right? that would be stupid.
+    if batch > int(messages):
+        raise PylogError('batch number larger than total amount of messages')
+    else:
+        lgr.debug('batch: {}'.format(batch))
+    instance, client = config_transport(
+        trans, transport + 'Transport', transport_config)
+    # send the stuff
+    send(instance, client, formatter + 'Formatter', formatter_config,
+         messages, gap, batch)
+    # maybe close a connection to the host is required...
     try:
         instance.close()
     except AttributeError:
@@ -310,5 +318,17 @@ def list_fake_types():
     print("\n".join(fakes_list))
 
 
+def list_transports():
+    for transport in dir(trans):
+        if 'Transport' in transport:
+            print('*** {}'.format(transport.replace('Transport', '')))
+
+
+def list_formatters():
+    for formatter in dir(forms):
+        if 'Formatter' in formatter:
+            print('*** {}'.format(formatter.replace('Formatter', '')))
+
+
 class PylogError(Exception):
     pass
