@@ -86,11 +86,11 @@ class AmqpTransport(BaseTransport):
     def __init__(self, config):
         try:
             self.host = config['host']
-            self.queue = config['queue']
-            self.exchange = config['exchange']
-            self.routing_key = config['routing_key']
         except KeyError as ex:
             raise('configuration not complete: {}'.format(ex))
+        self.queue = config.get('queue', DEFAULT_AMQP_QUEUE)
+        self.exchange = config.get('exchange', DEFAULT_AMQP_EXCHANGE)
+        self.routing_key = config.get('routing_key', DEFAULT_AMQP_ROUTING_KEY)
 
     def configure(self):
         if not self.host:
@@ -157,3 +157,25 @@ class StreamTransport(BaseTransport):
 
     def close(self):
         pass
+
+
+# class HTTPTransport(BaseTransport):
+#     """an http requests transport implementation"""
+#     def __init__(self, config):
+#         try:
+#             self.http = 'https' if config['https'] else 'http'
+#             self.request_type = config['request_type']
+#             self.host = config['host']
+#             self.port = config['port']
+#             self.uri = config['uri']
+#         except KeyError as ex:
+#             raise('configuration not complete: {}'.format(ex))
+
+#     def configure(self):
+#         import requests
+#         return requests
+
+#     def send(self, client, log):
+#         getattr(client, self.request_type)('{0}://{1}:{2}/{3}'.format(
+#             self.http, self.host, self.port, self.uri))
+#         client.get('https://api.github.com/user')
